function clf = OneClass_feedback(xi,yi,zi,nu,gamma)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	INPUTS:
%   xi: vector contaning normal
%   yi: vector contaning normal feedback
%   zi: vector contaning  outliers feedback
%   nu: fraction to be rejected (error on the target classs)
%   gamma: RBF kernel parameter
%
%   OUTPUTS:
%   clv: struct containing:
%           gamma: RBF kernel parameter
%           alpha: support vector weights
%           sv: support vectors
%           rho: threshold of decision function
%           alpha_index: support vectors index
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Julien Lesouple, Feb. 2020
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters
threshold=1e-5; %  threshold for alpha values

% About the data

nx = size(xi,1); % Dimension of normal training data
ny = size(yi,1); % Dimension of normal feedback (FA)
nz = size(zi,1); % Dimension of normal feedback (FA)
nxyz = nx+ny+nz;     % Total dimension of learning vectors

labelsx=ones(nx,1);
labelsy=2*ones(ny,1);
labelsz=-ones(nz,1);

labelsxyz=[labelsx; labelsy; labelsz]; % Stach the labels
xiyizi=[xi;yi;zi]; % Stack the values

C(1) = 1/(nx*nu(1)); % Optimisation parameter 1
C(2) = 1/(ny*nu(2)); % Optimisation parameter 2
C(3) = 1/(nz*nu(3)); % Optimisation parameter 2

K=exp(-gamma*squareform(pdist(xiyizi).^2));
f=zeros(nxyz,1);

% Linear equality constraints (alphas sum to 1)
Aeq = ones(1,nxyz);
beq = 1;

% Lower and upper bounds:
lb = zeros(nxyz,1);
lb(labelsxyz==-1) = -C(3);
ub = lb;
ub(labelsxyz==1) = C(1);
ub(labelsxyz==2) = C(2);
ub(labelsxyz==-1) = 0;

% Optimization procedure:
opt = optimset('quadprog');
opt.Algorithm = 'interior-point-convex';
opt.Display='off';

% Solve
alpha = quadprog(K,f,[],[],Aeq,beq,lb,ub,[],opt);
if (isempty(alpha))
    warning('No solution for the SVDD could be found');
    alpha = ones(nxyz,1)/nxyz;
end

Dx = alpha'*K;

% The support vectors:
alpha_index = find(abs(alpha)>threshold);

borderx = alpha_index(abs((alpha(alpha_index)-ub(alpha_index))) > threshold &(abs(alpha(alpha_index)) > threshold));

if (size(borderx,2)<1)
    borderx = alpha_index;
end

rho = mean(Dx(borderx));

% Set all non-support-vector alpha to 0
alpha(abs(alpha)<threshold) = 0.0;
alpha(abs(alpha-ub)< threshold)=ub(abs(alpha-ub)< threshold);

SVx = xiyizi(alpha_index,:); % support vectors = data with positive alpha

alpha = alpha(alpha_index);

clf.gamma = gamma;
clf.alpha = alpha;
clf.sv = SVx;
clf.rho=rho;
clf.alpha_index=alpha_index;
return

end



