function [clf,w0,w_memory,n_ano_nofb,n_ano_fb] = aad_ocsvm(xi,B,nu,ground_truth)
%AAD_OCSVM Performs AAD for One Class SVM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%	INPUTS:
%   xi: data
%   B: budget for feedback
%   nu: hyperparameters of OCSVM
%   ground_truth: expert feedback
%
%   OUTPUTS:
%   clf: struct containing:
%           gamma: RBF kernel parameter
%           alpha: support vector weights
%           sv: support vectors
%           rho: threshold of decision function
%           ano= anomalies presented to the expert
%           nom= nominal data presented to the expert
%   w0: solution opf OCSVM without expert feedback
%   w_memory: solution after each query to the expert
%   n_ano_nofb: number of anomalies presented to the expert after each
%               query without feedback
%   n_ano_fb: number of anomalies presented to the expert after each
%             query with feedback
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Julien Lesouple, Feb. 2020
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialization
w_memory=cell(1,B);
t=0;
sigma_xi= 1*median(pdist(xi));
gamma_xi=1/(2*sigma_xi^2); % Jaakkola
w0=OneClass_feedback(xi,[],[],nu,gamma_xi);
H_A=[];
H_N=[];
n_ano_fb=zeros(1,B);

K = exp(-w0.gamma*((pdist2(xi,w0.sv)).^2));
alpha=w0.alpha;
[~,ind_suspicious]=sort(K*alpha,'ascend');
ano=(ground_truth==-1);
top_ano=ano(ind_suspicious);
n_ano_nofb=cumsum(top_ano(1:B))';

ano_labeled=[];
nom_labeled=[];

while t<B
    disp([num2str(t/B*100),' %  '])
    t=t+1;
    score_t=K*alpha;
    [~,ind_sorted]=sort(score_t,'ascend');
    i_max=ind_sorted(1);
    
    if ground_truth(i_max)==-1
        H_A=[H_A;i_max];
        ano_labeled=[ano_labeled;xi(i_max,:)];
    elseif ground_truth(i_max)==1
        H_N=[H_N;i_max];
        nom_labeled=[nom_labeled;xi(i_max,:)];
    end
    
    xi(i_max,:)=[];
    ground_truth(i_max)=[];
    
    sigma_xi= 1*median(pdist([xi;nom_labeled]));
    gamma_xi=1/(2*sigma_xi^2); % Jaakkola
    
    w=OneClass_feedback(xi,nom_labeled,ano_labeled,nu,gamma_xi);

    K = exp(-w.gamma*((pdist2(xi,w.sv)).^2));
    alpha=w.alpha;
    w.ano=ano_labeled;
    w.nom=nom_labeled;
    w_memory{t}=w;
    n_ano_fb(t)=length(H_A);
end
clf=w;
end

